package jAnimation;

import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;
import javafx.animation.FadeTransition;
import javafx.animation.Interpolator;
import javafx.animation.PathTransition;
import javafx.animation.PathTransition.OrientationType;
import javafx.animation.Transition;
import javafx.geometry.Rectangle2D;
import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.Scene;
import javafx.scene.image.ImageView;
import javafx.scene.shape.CubicCurveTo;
import javafx.scene.shape.MoveTo;
import javafx.scene.shape.Path;
import javafx.util.Duration;

@ShortName("jAnimation")
@Author("Daestrum")
@Version(1.1f)
public class jAnimation extends Transition {

	private ImageView imageView;
	private int count;
	private int columns;
	private int offsetX;
	private int offsetY;
	private int width;
	private int height;
	private int lastIndex;
	private Rectangle2D rectangle;
	private Duration duration;
	public OrientationType PathOrientation_ORTHOGONAL_TO_TANGENT = PathTransition.OrientationType.ORTHOGONAL_TO_TANGENT;
	public OrientationType PathOrientation_NONE = PathTransition.OrientationType.NONE;
	/**
	 * @return ImageView
	 */
	public ImageView getImageView() {
		return imageView;
	}

	/**
	 * @param imageView
	 */
	public void setImageView(ImageView imageView) {
		this.imageView = imageView;
	}

	public int getCount() {
		return count;
	}

	/**
	 * @param count
	 * The number of animation frames in picture
	 */
	public void setCount(int count) {
		this.count = count;
	}

	/**
	 * @return int
	 * How many columns of animation frames in picture
	 */
	public int getColumns() {
		return columns;
	}

	public void setColumns(int columns) {
		this.columns = columns;
	}

	public int getOffsetX() {
		return offsetX;
	}

	public void setOffsetX(int offsetX) {
		this.offsetX = offsetX;
	}

	public int getOffsetY() {
		return offsetY;
	}

	public void setOffsetY(int offsetY) {
		this.offsetY = offsetY;
	}

	public int getWidth() {
		return width;
	}

	public void setWidth(int width) {
		this.width = width;
	}

	public int getHeight() {
		return height;
	}

	public void setHeight(int height) {
		this.height = height;
	}

	public int getLastIndex() {
		return lastIndex;
	}

	public void setLastIndex(int lastIndex) {
		this.lastIndex = lastIndex;
	}

	public int getDuration() {
		return (int) duration.toMillis();
	}

	/**
	 * @param duration
	 * number of milliseconds between frame changes
	 * too low a figure and image will be un-viewable
	 */
	public void setDuration(int duration) {
		this.duration = Duration.millis(duration);
	}

	public jAnimation() {

	}

	/**
	 * @param imageView
	 * @param duration
	 * @param count
	 * @param columns
	 * @param offsetX
	 * @param offsetY
	 * @param width
	 * @param height
	 * @return Animation object
	 */
	public jAnimation jInitialize(ImageView imageView, int duration,
			int count, int columns, int offsetX, int offsetY, int width,
			int height) {
		this.imageView = imageView;
		this.count = count;
		this.columns = columns;
		this.offsetX = offsetX;
		this.offsetY = offsetY;
		this.width = width;
		this.height = height;
		this.duration = Duration.millis(duration);

		this.setCycleDuration(this.duration);
		this.setInterpolator(Interpolator.LINEAR); 
		return this;
	}
	/**
	 * @param ox - offset x
	 * @param oy - offset y 
	 * @param w  - width of animation frame
	 * @param h  - height of animation frame
	 * @return viewport 
	 */
	public ImageView setViewPort(int ox,int oy,int w,int h){
			setRectangle(ox,oy,w,h);
			this.imageView.setViewport(this.rectangle);
			return this.imageView;
	}
	public ImageView setViewPortRect(ImageView i,Rectangle2D r){
		i.setViewport(r);
		return i;
	}
	public ImageView setScale(ImageView i,Double x,Double y){
		i.setScaleX(x);
		i.setScaleY(y);
		return i;
	}
	
	public void setRectangle(int ox,int oy,int w,int h){
		this.rectangle = new Rectangle2D(ox,oy,w,h);
	}
	/**
	 * @param ox - x  
	 * @param oy - y
	 * @param w  - width 
	 * @param h  - height
	 * @return Rectangle2D
	 */
	public Rectangle2D getRectangle(int x,int y,int w,int h){
		return new Rectangle2D(x , y, w, h);
	}
	/**
	 * @param id - name of group
	 * @return Group
	 */
	public Group newGroup(String id) {
		Group g = new Group();
		g.setId(id);
		return g;
	}
	
	/**
	 * @param s - scene
	 * @param g - group to add
	 */
	public void addGroup(Scene s,Group g){
		s.getRoot().getChildrenUnmodifiable().add(g);
	}
	/**
	 * @param g - Group to add to
	 * @param o - node to add to group
	 */
	public void addToGroup(Group g,Node o) {
		g.getChildren().add(o);
	}
	/**
	 * @param g - group 
	 * lists out members of the group
	 */
	public void showGroupMembers(Group g){
		System.out.println("Nodes for Group "+g.getId());
		for (Node n:g.getChildren()){
			System.out.println("\tNode> "+n.getId());
		}
	}
	/**
	 * @param n        	- node to act on
	 * @param from     	- float value start transparency 
	 * @param to	   	- float value end transparency	  
	 * @param duration	- how many millis to perform transition
	 * @param reverse	- should Fade reverse when complete
	 * @param cycle		- how many times to perform transition
	 * @return			- TadeTransition
	 */
	public FadeTransition setTransition(Node n,float from,float to,int duration,boolean reverse,int cycle){
		FadeTransition ft = new FadeTransition(Duration.millis(duration), n);
		ft.setFromValue(from);
		ft.setToValue(to);
		ft.setCycleCount(cycle);
		ft.setAutoReverse(reverse);
		return ft;
	}
	/**
	 * @param f	-	name of FadeTranisition to start
	 */
	public void startFade(FadeTransition f){
		f.play();
	}
	/**
	 * @param f	-	name of FadeTranisition to stop
	 */
	public void stopFade(FadeTransition f){
		f.stop();
	}
	/**
	 * @param f	-	name of FadeTranisition to pause
	 */
	public void pauseFade(FadeTransition f){
		f.pause();
	}
	/*
Path path = new Path();
path.getElements().add(new MoveTo(20,20));
path.getElements().add(new CubicCurveTo(380, 0, 380, 120, 200, 120));
path.getElements().add(new CubicCurveTo(0, 120, 0, 240, 380, 240));
PathTransition pathTransition = new PathTransition();
pathTransition.setDuration(Duration.millis(4000));
pathTransition.setPath(path);
pathTransition.setNode(rectPath);
pathTransition.setOrientation(PathTransition.OrientationType.ORTHOGONAL_TO_TANGENT);
pathTransition.setCycleCount(Timeline.INDEFINITE);
pathTransition.setAutoReverse(true);
pathTransition.play();
	 */
	public Path newPath(){
		return new Path();
	}
	public void addMoveTo(Path p,int x,int y){
		p.getElements().add(new MoveTo(x,y));
	}
	public void addCubicCurve(Path p,int x,int y,int x1,int y1,int x2,int y2){
		p.getElements().add(new CubicCurveTo(x,y,x1,y1,x2,y2));
	}
	public PathTransition newPathTransition(){
		return new PathTransition();
	}
	public PathTransition setPathTransition(Path p,int duration,Node node,int cycle,boolean reverse,PathTransition.OrientationType orient){
		PathTransition pt = new PathTransition();
		pt.setAutoReverse(reverse);
		pt.setCycleCount(cycle);
		pt.setDuration(Duration.millis(duration));
		pt.setPath(p);
		pt.setNode(node);
		pt.setOrientation(orient);
		return pt;
	}
	public void setPathDuration(PathTransition pt,int duration){
		pt.setDuration(Duration.millis(duration));
	}
	public void setPath(PathTransition pt,Path path){
		pt.setPath(path);
	}
	public void setPathTransitionNode(PathTransition pt,Node n){
		pt.setNode(n);
	}
	public void setOrientation(PathTransition pt,PathTransition.OrientationType type){
		pt.setOrientation(type);
	}
	public void setPathCycleCount(PathTransition pt,int x){
		pt.setCycleCount(x);
	}
	public void setPathAutoReverse(PathTransition pt,boolean reverse){
		pt.setAutoReverse(reverse);
	}
	public void startPathTransition(PathTransition pt){
		pt.play();
	}
	public void stopPathTransition(PathTransition pt){
		pt.stop();
	}
	public void pausePathTransition(PathTransition pt){
		pt.pause();
	}

	protected void interpolate(double k) {
		final int index = Math.min((int) Math.floor(k * count), count - 1);
		if (index != lastIndex) {
			final int x = (index % columns) * width + offsetX;
			final int y = (index / columns) * height + offsetY;
			this.imageView.setViewport(new Rectangle2D(x, y, width, height));
			lastIndex = index;
		}
	}

}
